/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package mv3302;

import simkit.Entity;

/**
 * The Ship class extends the Entity class from the Simkit library and
 * represents a ship object that can be used in a simulation. It keeps track of
 * the remaining time it takes to unload its cargo.
 *
 * @author dansl
 */
public final class Ship extends Entity {

    protected double remainingUnloadingTime;

    /**
     * Constructs a new Ship object with the specified remaining unloading time.
     *
     * @param remainingUnloadingTime the time remaining to unload the ship's
     * cargo.
     */
    public Ship(double remainingUnloadingTime) {
        super();
        setRemainingUnloadingTime(remainingUnloadingTime);
    }

    /**
     * Reduces the remaining unloading time of the ship based on the elapsed
     * time and the given rate.
     * @param rate the rate at which the ship is being unloaded.
     * @throws IllegalArgumentException if the rate is less than 0.
     */
    public void work(double rate) {
        if (rate < 0.0) {
            throw new IllegalArgumentException("Input Rate must be > 0.0: " + rate);
        }
        remainingUnloadingTime -= getElapsedTime() * rate;
    }

    /**
     * @return the remainingUnloadingTime
     */
    public double getRemainingUnloadingTime() {
        return remainingUnloadingTime;
    }

    /**
     * @param remainingUnloadingTime the remainingUnloadingTime to set
     */
    public void setRemainingUnloadingTime(double remainingUnloadingTime) {
        this.remainingUnloadingTime = remainingUnloadingTime;
    }
}
